/*

// BALLS 
// by Alma-nac & Ruairi Glynn
// DMX winch control 
// Code by Felix Faire (of Interactive Architecture Lab, Bartlett)
// Supported by DMX library of Francesco Anselmo (of Arup)

Navigate to the "Balls" tab above to change the height and colour
behaviour functions. Or feel free to look around the rest of the code to 
see how it works.

To automatically start and stop the reset process:

Start = press 'r'
Stop = press 't'

(if your computer/laptop does not have a default microphone the program may not run)

*/






// import external libraries
import processing.serial.*;
import ddf.minim.*;
import controlP5.*;

Minim minim;
AudioInput in;

// GUI control
ControlP5 cp5;

Accordion accordion;
Group gp1;
Group gp2;
Group gp3;
Group gp4;
ColorPicker cp1;
ColorPicker cp2;

Gradient grad;
Gradient day1;
Gradient day2;

// Enttec DMX USB PRO communication routines

static byte DMX_PRO_MESSAGE_START = byte(0x7E);
static byte DMX_PRO_MESSAGE_END = byte(0xE7);
static byte DMX_PRO_SEND_PACKET = byte(6);


// declare dmxHeight and dmxColor control
DMXUniverse dmxHeight;
DMXUniverse dmxLight;

int DMXmaxSpeed = 120;


// color variables
int colorMode = 1;
color c1 = color(0, 255, 0);
color c2 = color(0, 0, 255);


// boolean variables
boolean impulse = false;
boolean changing = false;
boolean resetting = false;

boolean waveNoise = false;
boolean clapped = false;
boolean sendDMX = true;
boolean animate = true;
boolean speedSet = false;

boolean drawing = true;

boolean pullDown = false;
boolean pushUp = false;


// 3D array for balls, rgb lights and height
int xSize = 7;
int ySize = 6;
color[][] ballC = new color[xSize][ySize];
float[][] ballZ = new float[xSize][ySize];
color[][] tempC = new color[xSize][ySize];
color[][] prevC = new color[xSize][ySize];
int[][] tempHeight = new int[xSize][ySize];

int[] allBallZ = new int[42];


// timing variables
float t = 0;
float rT = 0;
float sT = 0;
float cT = 0;
float longT = 0;
float timeSpeed = 0.003;
float resetT = 0;

float maxAmp = 255*0.5;
float tideLevel = 255*0.5;


// wave variables
float amp1 = 100;
float amp2 = 70;
float freq1 = 0.7;
float freq2 = 0.9;

float waveMultiplier = 1.0f;
float noiseMultiplier = 0.0f;

float ampMultiplier = 1.0f;
float freqMultiplier = 1.0f;

float heightOffset = 0;



//sound variables
float triggerVol = 30;
float aveVol = 0;
int numSamples = 100;
float[] aveSamples = new float[numSamples];
int sampleCount = 0;
float volumeScale = 700;
float volumeBoost = 0;

float extraNoise = 0;

ArrayList<Clap> claps = new ArrayList<Clap>();


PFont timeFont;
PFont font;

float x = 0;
float y = 0;
float z = 0;
float tx = 0;
float ty = 0;
float tz = 0;

int wolf = 0;


void setup() {
  size(1280, 720, OPENGL);
  background(0);
  smooth();

  timeFont  = loadFont("STHeiti-60.vlw");
  font  = loadFont("Tahoma-13.vlw");


  for (int i = 0; i < sampleCount; i++) {
    aveSamples[i] = 0;
  }


  // setup control
  cp5 = new ControlP5(this);
  cp5.setAutoDraw(false);

  grad = new Gradient();
  day1 = new Gradient();
  day2 = new Gradient();
  setupGradient();

  // setup audio inputs
  minim = new Minim(this);
  in = minim.getLineIn();

  // basic geometry drawing settings
  sphereDetail(7);
  rectMode(CORNERS);

  // setup DMX control
  for (int i = 0; i < Serial.list ().length; i++) {
    println("Opening: "+Serial.list()[i]);
  }

  // initialise the balls information to top with white
  for (int x = 0; x < xSize; x++) {
    for (int y = 0; y < ySize; y++) {

      // height
      ballZ[x][y] = 255;

      tempHeight[x][y] = 255;

      // rgb
      ballC[x][y] = color(255, 255, 255);

      tempC[x][y] = color(100);

      prevC[x][y] = color(100);

      allBallZ[x+7*y] = 255;
    }
  }
}




void draw() {
  background(0);

  if (frameCount == 1)   setupControl();

  // hourly reset and other functions
  dailyRoutine();

  // amplitude and frequency also modulate slowly over time to create a long repeatless pattern
  // amp1 is related to the average noise in the cafe

    amp1 = 10 + aveVol*10;
  amp2 = 40 + 20*sin(t*0.3);

  freq1 = 0.3 + 0.2*sin(t*0.05);
  freq2 = 0.4 + 0.3*sin(t*0.1 +0.5);

  tideLevel = (amp1+amp2);

  setHeight();
  setColor();

  updateTiming();

  if (drawing) {
    pushMatrix();
    translate(-xSize*40, -ySize*40, 0);

    pushMatrix();
    translate(tx, ty, tz);
    rotateX(x);
    rotateY(y);
    rotateZ(z);

    drawBalls();

    translate(xSize*40, ySize*40, 0);
    drawGrid();

    popMatrix();
    popMatrix();

    drawGUI();
  }

  drawAudio();

  if (resetting) reset();

  if (sendDMX) {
//    writeHeightToDMX();
//    writeColorsToDMX();
  }
}







// reset /////////////////////////////////////////////////



void reset() {

  // stops the animation and sends a constant up signal to the balls (channel 5)

  pushStyle();
  textSize(30);
  fill(255);
  textFont(timeFont);
  text("RESETTING TO 0", width*0.4, 200);
  popStyle();

  resetT += 0.01;

  if (sendDMX) {

    for (int x = 0; x < xSize; x++) {
      for (int y = 0; y < ySize; y++) {

        int hChannel = (x+y*xSize)*7;

        // sends a solid signal to the UP channel until reached the top
        //dmxHeight.setDMXChannel(hChannel+5, 200);
      }
    }
  }

  if (resetT > 50) {
    resetting = false;
    resetT = 0;
    animate = true;

    for (int x = 0; x < xSize; x++) {
      for (int y = 0; y < ySize; y++) {

        int hChannel = (x+y*xSize)*7;

        // cancel the up channel to complete the reset
        //dmxHeight.setDMXChannel(hChannel+5, 0);
      }
    }
  }
}








void resetWolfy() {
  wolf = wolf==200 ? 0 : 200;
  dmxHeight.setDMXChannel( (3+2*7)*7 + 5, wolf);
}

